import sys
import argparse
from collections import Counter
from dcicutils.ff_utils import get_metadata, patch_metadata
from functions import script_utils as scu

""" this is not currently used - was a mechanism to tag items with a particular 'release freeze' tag
    however, we don't currently freeze releases - could become useful at some point
"""

def get_args():
    parser = argparse.ArgumentParser(
        parents=[scu.create_ff_arg_parser()],
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument('reltag',
                        help="The release tag to query DataReleaseUpdates and add to items.")
    args = parser.parse_args()
    return args


def has_released(status):
    if 'released' in status:
        return True
    return False


def get_attype(res):
    ty = res.get('@type')
    if ty:
        ty = ty[0]
    return ty


def make_tag_patch(item, tag):
    if not scu.has_field_value(item, 'tags', tag):
        # not already tagged with this tag so make a patch and add 2 dict
        if item.get('tags'):
            tags = item['tags']
            tags.append(tag)
        else:
            tags = [tag]
        return {'tags': tags}
    return None


def do_patch(uid, type, patch, auth, dbupdate, cnts):
    if not dbupdate:
        print('DRY RUN - will update %s of type %s with %s' % (uid, type, patch))
        cnts['not_patched'] += 1
        return
    # import pdb; pdb.set_trace()
    res = patch_metadata(patch, uid, auth)
    # res = {'status': 'testing'}
    print('UPDATING - %s of type %s with %s' % (uid, type, patch))
    rs = res['status']
    print(rs)
    if rs == 'success':
        cnts['patched'] += 1
    else:
        cnts['errors'] += 1
        print(res)
    return


def add_tag2item(auth, iid, tag, seen, cnts, itype=None, dbupdate=False):
    # turns out that we do need to do a get as tags aren't embedded
    item = get_metadata(iid, auth)
    status = item.get('status')
    uid = item.get('uuid')
    if (not uid) or (uid in seen):
        print("SEEN OR IDLESS ITEM - SKIPPING")
        cnts['skipped'] += 1
        return
    seen.append(uid)
    if has_released(status):
        attype = get_attype(item)
        if not attype:
            attype = itype
        patch = make_tag_patch(item, tag)
        if patch:
            do_patch(uid, attype, patch, auth, dbupdate, cnts)
        else:
            print('NOTHING TO PATCH - skipping %s' % uid)
            cnts['skipped'] += 1
    else:
        print("STATUS %s doesn't get tagged - skipping %s" % (status, uid))
        cnts['skipped'] += 1
    return


def main():  # pragma: no cover
    args = get_args()
    dbupdate = args.dbupdate
    auth = scu.authenticate(key=args.key, keyfile=args.keyfile, env=args.env)

    cnts = Counter()
    reltag = args.reltag
    # build the search query string
    query = 'type=DataReleaseUpdate&update_tag=' + reltag
    relupdates = scu.get_item_ids_from_args([query], auth, True)
    update_items = []
    for u in relupdates:
        res = get_metadata(u, auth)
        for ui in res.get('update_items'):
            if ui.get('primary_id'):
                update_items.append(ui['primary_id'])
    seen = []
    # update_items = ['experiment-set-replicates/4DNESOI2ALTL']
    for item in update_items:
        res = get_metadata(item, auth)
        uid = res.get('uuid')
        type = get_attype(res)
        cnts[type] += 1
        if (not uid) or (uid in seen) or ('ExperimentSet' not in type):
            # case for first freeze (no processed files included)
            print("SKIPPING ", uid)
            cnts['skipped'] += 1
            continue
        add_tag2item(auth, uid, reltag, seen, cnts, type, dbupdate)

        if 'ExperimentSet' in type:
            # get the experiments and files
            exps = res.get('experiments_in_set')
            if exps is not None:
                cnts['Experiment'] += len(exps)
                for exp in exps:
                    # import pdb; pdb.set_trace()
                    add_tag2item(auth, exp, reltag, seen, cnts, 'Experiment', dbupdate)
                    files = exp.get('files')
                    if files is not None:
                        cnts['FileFastq'] += len(files)
                        for file in files:
                            file = add_tag2item(auth, file, reltag, seen, cnts, 'FileFastq', dbupdate)
                    epfiles = exp.get('processed_files')
                    # epfiles = None  # case for first freeze (no processed files included)
                    if epfiles is not None:
                        cnts['FileProcessed'] += len(epfiles)
                        for epf in epfiles:
                            add_tag2item(auth, epf, reltag, seen, cnts, 'FileProcessed', dbupdate)

            # check the processed files directly associated to the eset
            # pfiles = res.get('procesed_files')
            pfiles = None  # case for first freeze (no processed files included)
            if pfiles is not None:
                cnts['FileProcessed'] += len(pfiles)
                for pf in pfiles:
                    add_tag2item(auth, pf, reltag, seen, cnts, 'FileProcessed', dbupdate)
    print(cnts)


if __name__ == '__main__':  # pragma: no cover
    main()



#!/usr/bin/env python3
'''
Given a list of Repsets IDs and a title of the opf group
will transfer processed_files to other_processed_files
'''
import sys
import argparse
from dcicutils.ff_utils import get_authentication_with_server, get_metadata, patch_metadata
from functions import script_utils as scu


def get_args(args):
    parser = argparse.ArgumentParser(
        parents=[scu.create_input_arg_parser(), scu.create_ff_arg_parser()],
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument('--title',
                        default='Set of preliminary processed files',
                        help="The title for the group of other processed files.")
    args = parser.parse_args()
    return args


def main():  # pragma: no cover
    args = get_args(sys.argv[1:])


    print('#', auth.get('server'))
    id_list = scu.get_item_ids_from_args(args.input, auth, args.search)
    auth = scu.authenticate(key=args.key, keyfile=args.keyfile, env=args.env)
    for itemid in id_list:
        # get the existing data in other p
        item_data = get_metadata(itemid, auth, add_on='frame=raw')
        pfiles = item_data.get('processed_files')
        if not pfiles:
            continue
        patch_data = item_data.get('other_processed_files', [])
        if patch_data:
            # does the same title exist
            if args.title in [i['title'] for i in patch_data]:
                print(itemid, 'already has preliminary results')
                continue

        patch_data.append({'title': args.title,
                           'type': 'preliminary',
                           'files': pfiles})
        if patch_data:
            patch = {'other_processed_files': patch_data}
            if args.dbupdate:
                res = patch_metadata(patch, obj_id=itemid, key=auth, add_on='delete_fields=processed_files')
                print(res.get('status'))
            else:
                print("DRY RUN -- will patch")
                print(patch)
                print('and delete processed_files field value')


if __name__ == '__main__':  # pragma: no cover
    main()



#!/usr/bin/env python3

import sys
import argparse
import json
from datetime import datetime
from dcicutils.ff_utils import (
    get_authentication_with_server,
    get_metadata,
    patch_metadata,
    post_metadata,
)
from functions.script_utils import create_ff_arg_parser, authenticate


def get_args():  # pragma: no cover
    parser = argparse.ArgumentParser(
        description='Given a file of ontology term jsons (one per line) load into db',
        parents=[create_ff_arg_parser()],
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )

    parser.add_argument('infile',
                        help="the datafile containing object data to import")
    args = parser.parse_args()
    return args


def get_id(term):
    id_tag = term.get('uuid')
    if not id_tag:
        id_tag = term.get('term_id')
    if not id_tag:
        id_tag = term.get('term_name')
    return id_tag


def main():  # pragma: no cover
    start = datetime.now()
    print(str(start))
    args = get_args()
    auth = scu.authenticate(key=args.key, keyfile=args.keyfile, env=args.env)

    phase2 = {}
    # assumes a single line corresponds to json for single term
    if not args.dbupdate:
        print("DRY RUN - use --dbupdate to update the database")
    with open(args.infile) as terms:
        for t in terms:
            phase2json = {}
            term = json.loads(t)
            id_tag = get_id(term)
            if id_tag is None:
                print("No Identifier for ", term)
            else:
                tid = '/ontology-terms/' + id_tag
                # look for parents and remove for phase 2 loading if they are there
                if 'parents' in term:
                    phase2json['parents'] = term['parents']
                    del term['parents']
                if 'slim_terms' in term:
                    phase2json['slim_terms'] = term['slim_terms']
                    del term['slim_terms']

                try:
                    dbterm = get_metadata(tid, auth)
                except:  # noqa
                    dbterm = None
                op = ''
                if dbterm and 'OntologyTerm' in dbterm.get('@type', []):
                    if args.dbupdate:
                        e = patch_metadata(term, dbterm["uuid"], auth)
                    else:
                        e = {'status': 'dry run'}
                    op = 'PATCH'
                else:
                    if args.dbupdate:
                        e = post_metadata(term, 'OntologyTerm', auth)
                    else:
                        e = {'status': 'dry run'}
                    op = 'POST'
                status = e.get('status')
                if status and status == 'dry run':
                    print(op, status)
                elif status and status == 'success':
                    print(op, status, e['@graph'][0]['uuid'])
                    if phase2json:
                        phase2[e['@graph'][0]['uuid']] = phase2json
                else:
                    print('FAILED', tid, e)

    print("START LOADING PHASE2 at ", str(datetime.now()))
    for tid, data in phase2.items():
        if args.dbupdate:
            e = patch_metadata(data, tid, auth)
        else:
            e = {'status': 'dry run'}
        status = e.get('status')
        if status and status == 'dry run':
            print('PATCH', status)
        elif status and status == 'success':
            print('PATCH', status, e['@graph'][0]['uuid'])
        else:
            print('FAILED', tid, e)
    end = datetime.now()
    print("FINISHED - START: ", str(start), "\tEND: ", str(end))


if __name__ == '__main__':  # pragma: no cover
    main()



import sys
import argparse
from dcicutils import ff_utils as ff
from functions import script_utils as scu


def get_args():  # pragma: no cover
    parser = argparse.ArgumentParser(
        description='Provide a search query suffix and get a list of item uuids',
        parents=[scu.create_ff_arg_parser()],
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument('query',
                        help="A search string \
                        eg. type=Biosource&biosource_type=primary cell")
    args = parser.parse_args()
    return args


def main():
    args = get_args()
    auth = scu.authenticate(key=args.key, keyfile=args.keyfile, env=args.env)
    itemids = scu.get_item_ids_from_args([args.query], auth, True)
    for itemid in itemids:
        print(itemid)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

import sys
import argparse
from dcicutils.ff_utils import get_authentication_with_server, get_metadata, patch_metadata
from functions import script_utils as scu
""" DEPRECATED
    this script will take info from a biorxiv item with linked data and transfer to a publication that is in the db
    this has been incorporated into and been deprecated by the foursight check and action that does the same but better
    although there may be a bit more flexibility in using this script
"""


def get_args():  # pragma: no cover
    parser = argparse.ArgumentParser(
        parents=[scu.create_ff_arg_parser()],
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument('old',
                        help="The uuid or ID of the Biorxiv publication")
    parser.add_argument('new',
                        help="The uuid or ID of the Published Journal Article")
    parser.add_argument('--vals2skip',
                        nargs='+',
                        help="A list of values or IDs (uuids, accessions ...) to not transfer")

    args = parser.parse_args()
    return args


def remove_skipped_vals(val, vals2skip=None, auth=None):
    if not (val and vals2skip):
        return val
    is_string = False
    if isinstance(val, str):
        val = [val]
        is_string = True
    val = [v for v in val if v not in vals2skip]
    if auth:
        vuuids = [scu.get_item_uuid(v, auth) for v in val]
        skuids = [scu.get_item_uuid(v, auth) for v in vals2skip]
        for i, v in enumerate(vuuids):
            if v is not None:
                if v in skuids:
                    del val[i]
    if val:
        if is_string:
            return val[0]
        return val
    return None


def create_patch_for_new_from_old(old, new, fields2move, vals2skip=None):
    # build the patch dictionary
    ja_patch_dict = {}
    skipped = {}  # has info on skipped fields
    for f in fields2move:
        val = old.get(f)
        val = remove_skipped_vals(val, vals2skip)
        if val:
            jval = new.get(f)  # see if the field already has data in new item
            if jval:
                skipped[f] = {'old': val, 'new': jval}
                continue
            if isinstance(val, list):
                litems = []
                for v in val:
                    if isinstance(v, dict):
                        uid = v.get('uuid')
                        if uid is not None:
                            litems.append(uid)
                    else:
                        litems.append(v)
                if litems:
                    ja_patch_dict[f] = litems
            else:
                ja_patch_dict[f] = val
    return ja_patch_dict, skipped


def move_old_url_to_new_aka(biorxiv, jarticle, patch={}, skip={}):
    if 'url' in biorxiv:
        if 'aka' in jarticle:
            skip['aka'] = {'old': jarticle['aka'], 'new': biorxiv['url']}
        else:
            patch['aka'] = biorxiv['url']
    return patch, skip


def patch_and_report(auth, patch_d, skipped, uuid2patch, dryrun):
    # report and patch
    if dryrun:
        print('DRY RUN - nothing will be patched to database')
    if skipped:
        print('WARNING! - SKIPPING for', uuid2patch)
        for f, v in skipped.items():
            print('Field: %s\tHAS: %s\tNOT ADDED: %s' % (f, v['new'], v['old']))

    if not patch_d:
        print('NOTHING TO PATCH - ALL DONE!')
    else:
        print('PATCHING -', uuid2patch)
        for f, v in patch_d.items():
            print(f, '\t', v)

        if not dryrun:
            # do the patch
            res = patch_metadata(patch_d, uuid2patch, auth)
            if res['status'] == 'success':
                print("SUCCESS!")
                return True
            else:
                print("FAILED TO PATCH", uuid2patch, "RESPONSE STATUS", res['status'], res['description'])
                return False
    return True


def find_and_patch_item_references(auth, olduuid, newuuid, dryrun):
    search = "type=Item&references.uuid=" + olduuid
    itemids = scu.get_item_ids_from_args([search], auth, True)
    complete = True
    if not itemids:
        print("No references to %s found." % olduuid)
    for iid in itemids:
        ok = patch_and_report(auth, {'references': [newuuid]}, None, iid, dryrun)
        if not ok and complete:
            complete = False
    return complete


def main():  # pragma: no cover
    args = get_args()
    auth = scu.authenticate(key=args.key, keyfile=args.keyfile, env=args.env)
    dryrun = not args.dbupdate

    biorxiv = get_metadata(args.old, auth)
    jarticle = get_metadata(args.new, auth)

    if biorxiv.get('status') == 'error':
        print('Biorxiv record %s cannot be found' % args.old)
        sys.exit(1)
    if jarticle.get('status') == 'error':
        print('Journal Article record %s cannot be found' % args.new)
        sys.exit(1)
    # make sure we can get the uuid to patch
    juuid = jarticle.get('uuid')
    # build the patch dictionary
    fields2transfer = ['categories', 'exp_sets_prod_in_pub', 'exp_sets_used_in_pub', 'published_by']
    patch_dict, skipped = create_patch_for_new_from_old(biorxiv, jarticle, fields2transfer, args.vals2skip)
    patch_dict, skipped = move_old_url_to_new_aka(biorxiv, jarticle, patch_dict, skipped)

    # do the patch
    ok = patch_and_report(auth, patch_dict, skipped, juuid, dryrun)

    if not ok:
        sys.exit(1)  # bail out if initial transfer doesn't work

    # find items with reference to old paper
    buuid = biorxiv.get('uuid')
    complete = find_and_patch_item_references(auth, buuid, juuid, dryrun)
    if not complete:
        print("ALL REFERENCES POINTING TO %s NOT UPDATED - CHECK AND FIX!" % buuid)


if __name__ == '__main__':
    main()



import sys
import argparse
from dcicutils.ff_utils import get_metadata, patch_metadata
from functions import script_utils as scu
""" Script to add arbitrary tags to items that can be tagged
"""

def make_tag_patch(item, tag):
    if item.get('tags'):
        tags = item['tags']
        tags.append(tag)
    else:
        tags = [tag]
    return {'tags': tags}


def get_args():  # pragma: no cover
    parser = argparse.ArgumentParser(
        description='Add a tag to provided items (and optionally their children)',
        parents=[scu.create_input_arg_parser(), scu.create_ff_arg_parser()],
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument('tag',
                        help="The tag you want to add - eg. '4DN Standard'")
    parser.add_argument('--taglinked',
                        default=False,
                        action='store_true',
                        help='Tag items linked to items that are input')
    parser.add_argument('--types2exclude',
                        nargs='+',
                        help="List of Item Types to Explicitly Exclude Tagging - \
                        you may have some linked items that can get tags but may \
                        not want to tag them with this tag")
    args = parser.parse_args()
    return args


def main():  # pragma: no cover
    args = get_args()
    auth = scu.authenticate(key=args.key, keyfile=args.keyfile, env=args.env)
    itemids = scu.get_item_ids_from_args(args.input, auth, args.search)
    taggable = scu.get_types_that_can_have_field(auth, 'tags')
    if args.types2exclude is not None:
        # remove explicitly provide types not to tag
        taggable = [t for t in taggable if t not in args.types2exclude]

    seen = []   # only need to add tag once so this keeps track of what's been seen
    to_patch = {}   # keep track of those to patch
    # main loop through the top level item ids
    for itemid in itemids:
        items2tag = {}
        if args.taglinked:
            # need to get linked items and tag them
            linked = scu.get_linked_items(auth, itemid, {})
            items2tag = scu.filter_dict_by_value(linked, taggable, include=True)
        else:
            # only want to tag provided items
            itype = scu.get_item_type(auth, itemid)
            if itype in taggable:
                items2tag = {itemid: itype}
        for i, t in items2tag.items():
            if i not in seen:
                seen.append(i)
                item = get_metadata(i, auth)
                if not scu.has_field_value(item, 'tags', args.tag):
                    # not already tagged with this tag so make a patch and add 2 dict
                    to_patch[i] = make_tag_patch(item, args.tag)

    # now do the patching or reporting
    for pid, patch in to_patch.items():
        if args.dbupdate:
            pres = patch_metadata(patch, pid, auth)
            print(pres['status'])
        else:
            print("DRY RUN: patch ", pid, " with ", patch)


if __name__ == '__main__':  # pragma: no cover
    main()



import sys
import argparse
from dcicutils import ff_utils as ff
from functions import script_utils as scu


def get_args():  # pragma: no cover
    parser = argparse.ArgumentParser(
        description='Provide a search query suffix and get a list of item uuids',
        parents=[scu.create_ff_arg_parser(), scu.create_input_arg_parser()],
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    args = parser.parse_args()
    return args


def main():
    args = get_args()
    auth = scu.authenticate(key=args.key, keyfile=args.keyfile, env=args.env)
    print("Working on {}".format(auth.get('server')))
    itemids = scu.get_item_ids_from_args(args.input, auth, args.search)
    seen = []
    failed = []
    for itemid in itemids:
        print("Touching ", itemid)
        if args.dbupdate:
            try:
                res = ff.patch_metadata({}, itemid, auth)
                print(res.get('status'))
                if res.get('status') == 'success':
                    seen.append(itemid)
            except Exception:
                print(itemid, ' failed to patch')
                failed.append(itemid)
                continue
        else:
            print('dry run!')
    for i in seen:
        print(i)
    print("Failures")
    for f in failed:
        print(f)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
import sys
import argparse
import datetime
import re
from dcicutils.ff_utils import (
    patch_metadata, search_metadata,
    get_metadata)
from dcicwrangling.functions import script_utils as scu
from dcicwrangling.functions.notebook_functions import digest_xlsx, reader
'''
Parsing damid processed file worksheet to generate the various bins
of other processed files using information from the linked_dataset column
with the expectation that the linked_dataset will be an identifier for the
experiment or replicate set to which the file should be linked.

As the bin used for 'official' processed files - as opposed to supplementary
files can change depending on the target or experiment type (damid vs. pA-damid)
the PF_BIN specifies which bin to use

NOTE: there are variations that can deal with incomplete information - i.e. not
specifying the experiments but just using replicate set ids and then info in aliases
to match up replicates - see Andy if needed
'''

PF_BIN = '5kb bin'  # the bin for which a subset of files should be considered processed_files rather than opfs


def extract_rows(infile):
    book, sheets = digest_xlsx(infile)
    data = []
    row = reader(book, sheetname='FileProcessed')
    fields = next(row)
    fields = [f.replace('*', '') for f in fields]
    types = next(row)
    fields.pop(0)
    types.pop(0)
    for values in row:
        if values[0].startswith('#'):
            continue
        values.pop(0)
        meta = dict(zip(fields, values))
        data.append(meta)
    return data


def is_processed_bin(desc, meta):
    ''' Putting bam files and select files in bin specified by PF_BIN into the
        processed file bin this is a specific check for those attributes
    '''
    if 'mapped reads' in desc:
        return True
    if desc.startswith(PF_BIN):
        if (meta.get('file_type') == 'normalized counts' and meta.get('file_format') == 'bw') or (
                meta.get('file_type') == 'LADs' and meta.get('file_format') == 'bed'):
            return True
    return False


def create_patch(item, label, rep=None):
    patch = {}
    if rep:
        label = label + ' ' + rep
    item_pfs = item.get('processed_files')
    item_opfs = item.get('other_processed_files')
    if not (item_pfs or item_opfs):
        print("NO FILES FOR {}".format(item.get('uuid')))
        return
    if item_pfs:
        patch['processed_files'] = item_pfs
    if item_opfs:
        for bin, opfs in item_opfs.items():
            if bin == 'Other':
                opftitle = 'Other files - non-binned'
                opfdesc = 'Non-bin specific files for {}'.format(label)
            elif bin == PF_BIN:
                opftitle = 'Additional {}ned files'.format(bin)
                opfdesc = 'Additional files associated with the {} size processing of data for {}'.format(bin, label)
            else:
                opftitle = '{}ned files'.format(bin)
                opfdesc = 'The files associated with the {} size processing of data for {}'.format(bin, label)
            patch.setdefault('other_processed_files', []).append(
                {'title': opftitle, 'description': opfdesc, 'type': 'supplementary', 'files': opfs})
    return patch


def get_args(args):
    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter,
        parents=[scu.create_ff_arg_parser(), scu.create_input_arg_parser()],
    )
    args = parser.parse_args()
    return args


def main():  # pragma: no cover
    # initial set up
    args = get_args(sys.argv[1:])
    auth = scu.authenticate(key=args.key, keyfile=args.keyfile, env=args.env)

    repre = re.compile(r'_r\d+_')
    binre = re.compile(r'^\S+ bin')
    erepnore = re.compile(r'replicate\s\d+')

    # this if for parsing excel but could use fourfront query
    infile = args.input[0]
    query = None
    if len(args.input) > 1:
        query = args.input[1]

    metadata = extract_rows(infile)
    patch_items = {}
    seen_esets = {}  # if you are dealing with an experiment want to use the dataset_label and condition
    # of the replicate set to create the label
    # going row by row to add file to correct spot
    for meta in metadata:
        # checking if we have linked dataset info in sheet - should be either an
        # experiment set or experiment
        linked_dataset_id = meta.get('#linked datasets')
        if not linked_dataset_id:
            print("Can not get dataset_id for {}".format(meta))
            continue
        file_alias = meta.get('aliases')

        # build basic ds for the set
        if linked_dataset_id not in patch_items:
            item = get_metadata(linked_dataset_id, auth)  # either experiment or eset
            euuid = item.get('uuid')
            if not euuid:
                print("Can't get uuid for {} - skipping".format(linked_dataset_id))
                continue
            if 'experiments_in_set' in item:  # we've got an experiment set
                label = item.get('dataset_label') + ' ' + item.get('condition')
            else:  # we've got an experiment
                esets = item.get('experiment_sets')
                if len(esets) != 1:  # some sort of unusual situation
                    raise (Exception, 'experiment linked to multiple experiment sets -- abort!')
                esetid = esets[0].get('uuid')
                if esetid not in seen_esets:
                    eset = get_metadata(esetid, auth)
                    label = eset.get('dataset_label') + ' ' + eset.get('condition')
                    seen_esets[esetid] = label
                else:
                    label = seen_esets[esetid]

            patch_items[linked_dataset_id] = {'uuid': euuid, 'label': label, 'processed_files': [], 'other_processed_files': {}, 'experiments': {}}
        # use description to get replicate number if any and bin size if any
        desc = meta.get('description')

        bin = binre.match(desc)

        if is_processed_bin(desc, meta):
            patch_items[linked_dataset_id]['processed_files'].append(file_alias)
        else:
            if bin:
                bin = bin.group()
            else:
                bin = 'Other'
            patch_items[linked_dataset_id]['other_processed_files'].setdefault(bin, []).append(file_alias)

    patch_data = {}
    for e in patch_items.values():
        label = e.get('label')
        patch = create_patch(e, label)
        if patch:
            euid = e.get('uuid')
            existing_item = get_metadata(euid, auth)
            ipf = existing_item.get('processed_files')
            if ipf:
                if 'processed_files' in patch:
                    patch['processed_files'].extend(ipf)
            opf = existing_item.get('other_processed_files')
            if opf:
                if 'other_processed_files' in patch:
                    patch['other_processed_files'].extend(opf)

            patch_data[e.get('uuid')] = patch

    if patch_data:
        for puuid, pdata in patch_data.items():
            print(puuid, '\n', pdata, '\n\n')
            if args.dbupdate:
                try:
                    res = patch_metadata(pdata, puuid, auth)
                    print(res.get('status'))
                except Exception:
                    print("Can't patch {iid} with\n\t{p}".format(iid=puuid, p=pdata))


if __name__ == '__main__':
    main()



import sys
import argparse
from dcicutils.ff_utils import get_authentication_with_server, patch_metadata, delete_field
from functions import script_utils as scu


def get_args(args):
    parser = argparse.ArgumentParser(
        parents=[scu.create_input_arg_parser(), scu.create_ff_arg_parser()],
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument('field',
                        help="The field to update.")
    parser.add_argument('value',
                        help="The value(s) to update. Array fields need \"''\" surround \
                        even if only a single value i.e. \"'value here'\" or \"'v1' 'v2'\"")
    parser.add_argument('--isarray',
                        default=False,
                        action='store_true',
                        help="Field is an array.  Default is False \
                        use this so value is correctly formatted even if only a single value")
    parser.add_argument('--numtype',
                        help="options: 'i' or 'f' If the field value is integer or number deal accordingly")
    args = parser.parse_args(args)
    return args


def main():
    args = get_args(sys.argv[1:])
    auth = scu.authenticate(key=args.key, keyfile=args.keyfile, env=args.env)
    print("Working on {}".format(auth.get('server')))
    itemids = scu.get_item_ids_from_args(args.input, auth, args.search)
    field = args.field
    val = args.value
    if val == 'True':
        val = True
    elif val == 'False':
        val = False
    if args.isarray:
        val = [v for v in val.split("'") if v]
    ntype = args.numtype
    if ntype:
        if ntype == 'i':
            val = int(val)
        elif ntype == 'f':
            val = float(val)
    for iid in itemids:
        print("PATCHING", iid, "to", field, "=", val)
        if (args.dbupdate):
            # do the patch
            if val == '*delete*':
                res = delete_field(iid, field, auth)
            else:
                res = patch_metadata({args.field: val}, iid, auth)
            if res['status'] == 'success':
                print("SUCCESS!")
            else:
                print("FAILED TO PATCH", iid, "RESPONSE STATUS", res['status'], res['description'])
                # print(res)


if __name__ == '__main__':  # pragma:nocover
    main()



#!/usr/bin/env python3
'''
Given a list of item IDs will fetch the items or the fields of those items
specified in the --fields parameter)
'''
import argparse
from dcicutils.ff_utils import get_metadata
from functions import script_utils as scu


def get_args():
    parser = argparse.ArgumentParser(
        parents=[scu.create_input_arg_parser(), scu.create_ff_arg_parser()],
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument('--fields',
                        nargs='+',
                        help="The item fields to retrieve/report.")
    parser.add_argument('--noid',
                        action='store_true',
                        default='False',
                        help="By default the id provided is the first column of output - this flag removes that column")

    return parser.parse_args()


def main():  # pragma: no cover
    args = get_args()
    auth = scu.authenticate(key=args.key, keyfile=args.keyfile, env=args.env)

    print('#', auth.get('server'))
    id_list = scu.get_item_ids_from_args(args.input, auth, args.search)
    if args.fields:
        fields = args.fields

        header = '#id\t' + '\t'.join(fields)
        if args.noid is True:
            header = header.replace('#id\t', '#')
        print(header)
    problems = []
    for iid in id_list:
        try:
            res = get_metadata(iid, auth, add_on='frame=object')
        except Exception:
            problems.append(iid)
            continue

        if args.fields:
            line = ''
            # counts = {}
            for f in fields:
                val = res.get(f)
                # if val is not None:  # added in for specific use case
                if isinstance(val, dict):
                    val = val.get('uuid')
                elif isinstance(val, list):
                    # counts[f] = len(val)  # added in for specific use case
                    # if len(counts) > 1:
                    #     print(iid, '\t', counts)
                    # else:
                    #     cnt = list(counts.values())[0]
                    #     if cnt > 1:
                    #         print(iid, '\t', cnt)
                    vs = ''
                    for v in val:
                        if isinstance(v, dict):
                            v = v.get('uuid')
                        else:
                            v = str(v)
                        vs = vs + v + ', '
                    val = vs
                    if val.endswith(', '):
                        val = val[:-2]
                line = line + str(val) + '\t'
            if args.noid == 'False':
                line = iid + '\t' + line
            print(line)
        else:
            if args.noid is True:
                print(res)
            else:
                print(iid, '\t', res)
    if problems:
        print('THERE WAS A PROBLEM GETTING METADATA FOR THE FOLLOWING:')
        for p in problems:
            print(p)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

import sys
import argparse
import json
from datetime import datetime
from uuid import uuid4
from dcicutils.ff_utils import (
    get_authentication_with_server,
    post_metadata,
    get_metadata,
    patch_metadata,
)
from functions.script_utils import create_ff_arg_parser, authenticate
''' Will attempt to load data from a file into the database using the load_data endpoint if it can
    or post/patch_metadata if not
    The file can be a simple list of json items in which case you need to specify an item type
    with the --itype option (file created by generate_ontology is like this) or the file can
    specify a dictionary with item types as keys and list of jsons as values.

    If the --as_file option is used the json items must contain a uuid and the endpoint will
    attempt to read the file from the request - no ordering and if there are dependencies to
    linked items those items must either already be loaded or present in the file
    WARNING: currently only works locally or if file is uploaded as part of the app file system
'''

ORDER = [
    'user',
    'award',
    'lab',
    'static_section',
    'higlass_view_config',
    'page',
    'ontology',
    'ontology_term',
    'file_format',
    'badge',
    'organism',
    'genomic_region',
    'gene',
    'bio_feature',
    'target',
    'imaging_path',
    'publication',
    'publication_tracking',
    'document',
    'image',
    'vendor',
    'construct',
    'modification',
    'protocol',
    'sop_map',
    'biosample_cell_culture',
    'individual_human',
    'individual_mouse',
    'individual_fly',
    'individual_chicken',
    'biosource',
    'antibody',
    'enzyme',
    'treatment_rnai',
    'treatment_agent',
    'biosample',
    'quality_metric_fastqc',
    'quality_metric_bamqc',
    'quality_metric_pairsqc',
    'quality_metric_dedupqc_repliseq',
    'quality_metric_chipseq',
    'quality_metric_atacseq',
    'microscope_setting_d1',
    'microscope_setting_d2',
    'microscope_setting_a1',
    'microscope_setting_a2',
    'file_fastq',
    'file_processed',
    'file_reference',
    'file_calibration',
    'file_microscopy',
    'file_set',
    'file_set_calibration',
    'file_set_microscope_qc',
    'file_vistrack',
    'experiment_hi_c',
    'experiment_capture_c',
    'experiment_repliseq',
    'experiment_atacseq',
    'experiment_chiapet',
    'experiment_damid',
    'experiment_seq',
    'experiment_tsaseq',
    'experiment_mic',
    'experiment_set',
    'experiment_set_replicate',
    'data_release_update',
    'software',
    'analysis_step',
    'workflow',
    'workflow_mapping',
    'workflow_run_sbg',
    'workflow_run_awsem'
]


def get_args():  # pragma: no cover
    parser = argparse.ArgumentParser(
        description='Given a file of item jsons try to load into database',
        parents=[create_ff_arg_parser()],
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )

    parser.add_argument('infile',
                        help="the datafile containing json formatted items")
    parser.add_argument('--itypes',
                        nargs='*',
                        help="The item type(s) to load if not specified in the file by store key(s)")
    parser.add_argument('--id-field',
                        help="Field name to used as identifier for items (all item types in file)")
    parser.add_argument('--as-file',
                        default=False,
                        action='store_true',
                        help="Will attempt to load and process the file directly in the request. "
                             "This currently only works locally or if the file has been uploaded to "
                             "the apps file system")
    args = parser.parse_args()
    return args


def patch_jsons(auth, to_patch):
    for item in to_patch:
        uid = item.get('uuid')
        try:
            patch_metadata(item, uid, auth)
        except Exception as e:
            print(e)


def load_json(auth, itype, item_list, chunk_size=50):
    list_length = len(item_list)
    curr_pos = 0
    while curr_pos < list_length:
        slice_for = chunk_size if (chunk_size and chunk_size <= (list_length - curr_pos)) else list_length - curr_pos
        new_end = curr_pos + slice_for
        chunk = item_list[curr_pos: new_end]
        store = {itype: chunk}
        payload = {'store': store, 'overwrite': True}
        if 'localhost' in auth.get('server', ''):
            payload['config_uri'] = 'development.ini'
        try:
            post_metadata(payload, 'load_data', auth)
        except Exception as e:
            print("PROBLEM WITH POST")
            print(e)
        curr_pos = new_end


def load_file(auth, itype, filename):
    payload = {'in_file': filename, 'overwrite': True, 'itype': itype}
    if 'localhost' in auth.get('server', ''):
        payload['config_uri'] = 'development.ini'
    try:
        post_metadata(payload, 'load_data', auth)
    except Exception:
        raise


def get_item(val, auth):
    try:
        return get_metadata(val, auth).get('uuid')
    except:
        return None


def check_for_existing(item, itype, idfields, auth):
    uid = None
    for ifield in idfields:
        id2chk = item.get(ifield)
        if id2chk:
            if ifield == 'aliases':
                for a in id2chk:
                    uid = get_item(a, auth)
                    if uid:
                        return uid
            else:
                chkid = itype + '/' + id2chk
                uid = get_item(chkid, auth)
                if uid:
                    return uid
    return uid


def main():  # pragma: no cover
    start = datetime.now()
    print(str(start))
    args = get_args()
    auth = scu.authenticate(key=args.key, keyfile=args.keyfile, env=args.env)
    print('working on ', auth.get('server'))
    if args.as_file:
        if not args.dbupdate:
            print("DRY RUN - use --dbupdate to update the database")
        else:
            try:
                load_file(auth, args.itype, args.infile)
            except Exception as e:
                print(e)
    else:
        with open(args.infile) as ifile:
            item_store = json.loads(ifile.read())
            if not args.itype:
                if not isinstance(item_store, dict):
                    print("File is not in correct format")
                    sys.exit(1)
            else:
                if not isinstance(item_store, list):
                    print("File is not in correct format")
                    sys.exit(1)
                item_store = {args.itype: item_store}
            for itype, items in sorted(item_store.items(), key=lambda x: ORDER.index(x[0])):
                if not args.dbupdate:
                    print('DRY RUN - would try to load {} {} items'.format(len(items), itype))
                    continue
                if args.id_field:
                    identifiers = [args.id_field]
                else:
                    schema_path = 'profiles/' + itype + '.json'
                    schema_info = get_metadata(schema_path, auth)
                    identifiers = schema_info.get('identifyingProperties')
                # checking to see if an item exists
                # if no can use load_data endpoint
                # if yes do it the old fashioned way
                to_patch = []
                to_post = []
                for item in items:
                    uid = item.get('uuid')
                    if uid:
                        exists = get_item(uid, auth)
                        if exists:  # try a patch
                            to_patch.append(item)
                        else:
                            to_post.append(item)
                    else:
                        uid = check_for_existing(item, itype, identifiers, auth)
                        if uid:  # try a patch
                            item['uuid'] = uid
                            to_patch.append(item)
                        else:
                            uid = str(uuid4())
                            item['uuid'] = uid
                            to_post.append(item)
                if to_post:
                    load_json(auth, itype, to_post, chunk_size=1000)
                if to_patch:
                    patch_jsons(auth, to_patch)
    stop = datetime.now()
    print(str(stop))


if __name__ == '__main__':  # pragma: no cover
    main()



import sys
import argparse
from datetime import datetime
from dcicutils.ff_utils import get_authentication_with_server, get_metadata, post_metadata, patch_metadata
from functions import script_utils as scu
'''Generate provenance workflow_runs for processed files using the
    information in the 'produced_from' field.
    input is a list of file ids or a search for the files.
'''


def get_args(args):
    parser = argparse.ArgumentParser(
        parents=[scu.create_ff_arg_parser(), scu.create_input_arg_parser()],
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument('--workflow',
                        default="bef50397-4d72-4ed1-9c78-100e14e5c47f",
                        help="The uuid of the workflow to use to generate the run \
                              default is File Provenance Workflow"
                        )
    parser.add_argument('--omit_note',
                        default=False,
                        action='store_true',
                        help="Boolean to skip adding a note to tsv about non-DCIC processing \
                              default is False")

    args = parser.parse_args(args)
    return args


def _filter_none(my_list):
    nl = [li for li in my_list if li is not None]
    if len(nl) != len(my_list):
        print('WARNING: None values found in your list')
    return nl


def get_attribution(items):
    award = lab = None
    for i in items:
        award = i.get('award')
        lab = i.get('lab')
        if award and lab:
            try:
                award = award.get('uuid')
                lab = lab.get('uuid')
            except AttributeError:
                pass
            return award, lab
    # default to 4dn-dcic award and lab
    return 'b0b9c607-f8b4-4f02-93f4-9895b461334b', '828cd4fe-ebb0-4b36-a94a-d2e3a36cc989'


def create_wfr_meta_only_json(auth, workflow, inputs, outputs, alias=None, description=None):
    '''provide input file(s), output file(s), optional alias,
       optional description builds a metadata only workflow_run json
       currently designed for file tracing so does not do quality metrics
       or deal with arguments other than Input file and Output processed file'''
    workflow = scu.get_item_if_you_can(auth, workflow)
    if workflow.get('uuid') is None:
        return None
    infiles = _filter_none([scu.get_item_if_you_can(auth, i) for i in inputs])
    outfiles = _filter_none([scu.get_item_if_you_can(auth, o) for o in outputs])
    now = str(datetime.now())
    if alias is None:
        alias = '4dn-dcic-lab:' + workflow.get('name') + '_run_' + now.replace(':', '-').replace(' ', '-')
    wfr_title = workflow.get('title') + ' run on ' + now

    award, lab = get_attribution(outfiles + infiles + [workflow])

    wfr_json = {
        'workflow': workflow.get('uuid'),
        'aliases': [alias],
        'award': award,
        'lab': lab,
        'status': 'in review by lab',
        'title': wfr_title,
        'run_status': 'complete',
        'metadata_only': True
    }

    if description is not None:
        wfr_json['description'] = description

    args = workflow.get('arguments')
    input_files = []
    output_files = []
    if args:
        for arg in args:
            argname = arg.get('workflow_argument_name')
            if arg.get('argument_type') == 'Input file':
                # build the needed inputs
                for i, inf in enumerate(infiles):
                    input_files.append(
                        {
                            'workflow_argument_name': argname,
                            'value': inf.get('uuid'),
                            'ordinal': i + 1
                        }
                    )
            if arg.get('argument_type') == 'Output processed file':
                for outf in outfiles:
                    output_files.append(
                        {
                            'workflow_argument_name': argname,
                            'value': outf.get('uuid'),
                            'workflow_argument_format': outf.get('file_format').get('uuid'),
                            'type': 'Output processed file'
                        }
                    )

    if input_files:
        wfr_json['input_files'] = input_files
    if output_files:
        wfr_json['output_files'] = output_files
    return wfr_json


def add_notes_to_tsv(file_meta, auth):
    """ adds a notes to tsv with the canned value below to the processed file
        returns success, error or skip if the value already exists
    """
    note_txt = "This file contains processed results performed outside of the 4DN-DCIC standardized pipelines. The file and the information about its provenance, i.e. which files were used as input to generate this output was provided by or done in collaboration with the lab that did the experiments to generate the raw data. For more information about the specific analysis performed, please contact the submitting lab or refer to the relevant publication if available."
    n2tsv = file_meta.get('notes_to_tsv', [])
    for note in n2tsv:
        if note_txt in note:
            return "SKIP"
    n2tsv.append(note_txt)
    patch = {'notes_to_tsv': n2tsv}
    try:
        pres = patch_metadata(patch, file_meta.get('uuid'), auth)
    except Exception as e:
        print(e)
        return "ERROR"
    if pres.get('status') == 'success':
        return "SUCCESS"
    return "ERROR"


def main():
    args = get_args(sys.argv[1:])
    auth = scu.authenticate(key=args.key, keyfile=args.keyfile, env=args.env)
    dryrun = not args.dbupdate

    file_list = scu.get_item_ids_from_args(args.input, auth, args.search)
    wf_data = get_metadata(args.workflow, auth)
    for f in file_list:
        file_info = get_metadata(f, auth)
        parents = file_info.get('produced_from')
        if parents:
            inputs = []
            for p in parents:
                inputs.append(get_metadata(p, auth))
            wfr_json = create_wfr_meta_only_json(auth, wf_data, inputs, [file_info])
            if dryrun:
                print('DRY RUN -- will post')
                print(wfr_json)
            else:
                res = post_metadata(wfr_json, 'workflow_run_awsem', auth)
                print(res)
                if not args.omit_note:
                    # and add a notes_to_tsv to the file
                    patchstatus = add_notes_to_tsv(file_info, auth)
                    print(patchstatus)


if __name__ == '__main__':  # pragma: no cover
    main()



#!/usr/bin/env python3
# -*- coding: latin-1 -*-

import argparse
import os
import re
import requests
import sys
import time
from statistics import mean
import xml.etree.ElementTree as ET
import openpyxl
from functions.notebook_functions import list_column_titles, copy_xlsx_sheet
from Bio import Entrez
from urllib.error import HTTPError
import GEOparse


description = '''
Script for fetching metadata from GEO and inserting it into a Submit4dn metadata workbook.

Note1: Use of NCBI's Entrez querying system requires an email address.
There will be a prompt to enter an email address when this script is run.

Note2: Currently only works if each experiment in GEO has an SRA record. Will not
work properly for sequences in dbgap, but hope to implement this in the future.

'''


epilog = '''
Example usage:

$ python scripts/geo2fdn.py GSE93431 -i hic_rnaseq_workbook.xlsx -o GSE93431_metadata.xlsx

$ python scripts/geo2fdn.py GSE68992 -i hic_workbook.xlsx -o GSE68992_metadata.xlsx -t 'dnase hic'

'''


class Experiment:

    def __init__(self, exptype, instr, geo, title, biosample, link):
        self.exptype = exptype.lower()  # experiment type
        self.instr = instr[0] if len(instr) == 1 else instr  # sequencing instrument
        self.layout = ''  # single or paired end
        self.geo = geo  # geo accession starting with GSM
        self.title = title[0] if len(title) == 1 else title
        self.runs = []  # list of SRA accessions starting with SRR
        self.length = ''  # mean read length
        # self.study_title = study_title
        self.bs = biosample
        self.link = link
        self.public = True

    def get_sra(self):
        # look up SRA record to fill out more attributes
        print('Fetching SRA Record...')
        handle = handle_timeout(Entrez.efetch(db="sra", id=self.link))
        try:
            record = ET.fromstring(handle.readlines()[2])
        except Exception:
            try:
                handle = handle_timeout(Entrez.efetch(db="sra", id=self.link))
                text = handle.read()
                record = ET.fromstring(text[text.index('<EXPERIMENT_PACKAGE>'):text.index('</EXPERIMENT_PACKAGE_SET>')])
            except Exception:
                print("Couldn't parse {} from {}".format(self.link, self.geo))
                return
        if record.tag == 'ERROR':
            if 'is not public' in record.text:
                print(record.text + ' - will not be written to file!')
                self.public = False
            else:
                print('Could not get SRA record {}'.format(self.link))
            return
        lengths = [int(float(item.get('average'))) for item in record.iter('Read') if item.get('count') != '0']
        if lengths:
            self.length = int(mean(lengths))
        self.st = record.find('STUDY').find('DESCRIPTOR').find('STUDY_TITLE').text
        for item in record.find('EXPERIMENT').find('DESIGN').find('LIBRARY_DESCRIPTOR').find('LIBRARY_LAYOUT'):
            self.layout = item.tag.lower()
            break
        self.runs = [item.get('accession') for item in record.find('RUN_SET').findall('RUN')]


class Biosample:

    def __init__(self, acc, organism, description):
        self.acc = acc  # BioSample accession starting with SAMN
        self.organism = organism
        self.description = description
        # self.treatments = treatments


class Dataset:

    def __init__(self, gse, ids, experiments, biosamples):
        self.gse = gse  # GEO Series accession starting with GSE
        self.ids = ids  # GEO sample ids associated with series
        self.experiments = experiments  # list of experiment objects
        self.biosamples = biosamples  # list of biosample objects


valid_types = ['hic', 'hicseq', 'dnase hic', 'rnaseq', 'tsaseq', 'chipseq',
               'dna sprite', 'dnarna sprite', 'rnadna sprite', 'capturec',
               'repliseq', 'atacseq', 'damid', 'damidseq', 'chiapet', 'placseq']


type_dict = {'chipseq': 'ChIP-seq', 'tsaseq': 'TSA-seq', 'rnaseq': 'RNA-seq',
             'atacseq': 'ATAC-seq', 'capturec': 'capture Hi-C', 'damid': 'DAM-ID seq',
             'damidseq': 'DAM-ID seq', 'chiapet': 'ChIA-PET', 'placseq': 'PLAC-seq',
             'dnase hic': 'DNase Hi-C', 'dna sprite': 'DNA SPRITE',
             'dnarna sprite': 'RNA-DNA SPRITE', 'rnadna sprite': 'RNA-DNA SPRITE',
             'repliseq': 'Repli-seq'}


def handle_timeout(command):  # pragma: no cover
    '''
    To retry commands if the server connection times out.
    '''
    time.sleep(0.5)
    try:
        result = command
    except HTTPError:
        time.sleep(3)
        try:
            result = command
        except HTTPError:
            time.sleep(10)
            try:
                result = command
            except HTTPError:
                time.sleep(10)
                result = command
    return result


def parse_gsm(gsm, experiment_type=None):
    '''
    Parses information about individual experiment. Input is a GEOparse.gsm object.
    Function creates an Experiment object; if GSM record has an associated SRA record,
    it will also look up the SRA record and fill out more attributes.
    '''
    # if experiment is a microarray, don't parse
    if 'SRA' not in gsm.metadata['type']:
        return
    exp_type = experiment_type if experiment_type else gsm.metadata['library_strategy'][0]
    if not exp_type or exp_type.lower() == 'other':
        for item in gsm.metadata['data_processing']:
            if item.lower().startswith('library strategy'):
                exp_type = item[item.index(':') + 2:]
    if not exp_type or exp_type.lower() == 'other':
        title = re.sub('-', '', ', '.join(gsm.metadata['title']).lower())
        types = [item for item in valid_types if item in title]
        if len(types) == 1 or (types and all('hic' in t for t in types)):
            exp_type = types[-1]
    link = None
    for item in gsm.metadata['relation']:
        # get biosample relation
        if item.startswith('BioSample:'):
            bs = item[item.index('SAMN'):]
        # get SRA relation
        elif item.startswith('SRA:'):
            link = item[item.index('SRX'):]
    exp = Experiment(re.sub('-', '', exp_type.lower()), gsm.metadata['instrument_model'],
                     gsm.name, gsm.metadata['title'], bs, link)
    if link:  # if no SRA relation is in GSM metadata, sequencing data might be in dbgap
        exp.get_sra()  # get more metadata about sequencing runs
    return exp


def get_geo_metadata(acc, experiment_type=None):
    '''
    Parses information associated with a GEO Series or single experiment.
    Uses GEOparse library which downloads records from NCBI ftp rather than using
    NCBI Entrez e-utils, resulting in a single request rather than many. This
    function will parse information from the files and then delete them. Returns
    a Dataset object, holding information about all the associated experiments
    and biosamples.
    '''
    if acc.startswith('GSE') or '/GSE' in acc:  # experiment series
        if '/' in acc:
            gse = GEOparse.get_GEO(filepath=acc)
        else:
            gse = GEOparse.get_GEO(geo=acc)  # pragma: no cover
        # create Experiment objects from each GSM file
        experiments = [obj for obj in [parse_gsm(gsm, experiment_type) for gsm in gse.gsms.values()] if
                       obj and obj.public]
        # delete file after GSMs are parsed
        if '/' not in acc:  # pragma: no cover
            print('GEO parsing done. Removing downloaded soft file.')
            os.remove(f'{acc}_family.soft.gz')
        if not experiments:
            print('Sequencing experiments not found. Exiting.')
            return
        gds = Dataset(acc, gse.metadata['sample_id'], experiments,
                      [parse_bs_record(experiment.bs) for experiment in experiments])
        return gds
    elif acc.startswith('GSM') or '/GSM' in acc:  # single experiment
        if '/' in acc:
            gsm = GEOparse.get_GEO(filepath=acc)
        else:
            gsm = GEOparse.get_GEO(geo=acc)  # pragma: no cover
        exp = parse_gsm(gsm, experiment_type)
        print("GEO parsing done. Removing downloaded soft file.")
        try:
            os.remove(f'{acc}.txt')  # delete file after GSM is parsed
        except Exception:
            pass
        if not exp:
            print("Accession not a sequencing experiment, or couldn't be parsed. Exiting.")
            return
        gds = Dataset(None, [acc], [exp], [parse_bs_record(exp.bs)])
        return gds
    else:
        print('Input not a valid GEO accession.')
        return


def parse_bs_record(bs_acc):
    '''
    Takes in a BioSample accession, fetches the BioSample record, and
    parses it into a Biosample object.
    '''
    print("Fetching BioSample record...")
    bs_handle = handle_timeout(Entrez.efetch(db='biosample', id=bs_acc))
    bs_xml = ET.fromstring(bs_handle.read())
    atts = {}
    descr = ''
    acc = bs_xml.find('./BioSample').attrib['accession']
    org = [item.text for item in bs_xml.iter("OrganismName")]
    # turn org items such as "Mus musculus x Mus spretus" to Mus musculus
    org = ['Mus musculus' if 'mus musculus' in item.lower() else item for item in org]
    if not org:
        org = [item.attrib['taxonomy_name'] for item in bs_xml.iter("Organism")]
    # except IndexError:
    #     print("{} - can't get OrganismName".format(bs_acc))
    for item in bs_xml.iter("Attribute"):
        atts[item.attrib['attribute_name']] = item.text

    for name in ['source_name', 'sample_name', 'gender', 'strain', 'genotype', 'cross',
                 'cell_line', 'cell line', 'cell lines', 'tissue', 'sirna transfected', 'treatment', 'activation time']:
        if name in atts.keys() and atts[name].lower() != 'none':
            if atts[name] not in descr:
                descr += name + ": " + atts[name] + ' | '
            if name == 'treatment':
                treatments = atts[name]
                if not sum([term in treatments.lower() for term in ['blank', 'none', 'n/a']]):
                    # print message to indicate that Treatment tab will need to be filled
                    print(f"BioSample accession {acc} has treatment attribute",
                          "but treatment not written to file")
    descr = descr.rstrip(' | ')
    bs = Biosample(acc, org[0], descr)
    return bs


def get_geo_tables(geo_acc, outf, lab_alias='4dn-dcic-lab', email='', types=type_dict):
    '''
    Creates 3 separate tsv files containing information for fastq files,
    experiments, and biosamples associated with a GEO accession. Can be used if a
    blank workbook with the required Experiment sheets hasn't been created.

    Parameters:
    geo_acc - GEO accession (e.g. 'GSE93431')
    lab_alias - alias prefix; default is '4dn-dcic-lab'
    outf - prefix for output files. Output files will be named
           <outf>_expts.tsv, <outf>_fqs.tsv, and <outf>_bs.tsv.
    email - email to be supplied for NCBI Entrez e-utils.
    types - dictionary of experiment types - do not use, leave default.
    '''
    Entrez.email = email if email else input('Enter email address to use NCBI Entrez: ')
    gds = get_geo_metadata(geo_acc, experiment_type=None)
    with open(outf + '_expts.tsv', 'w') as outfile:
        for exp in gds.experiments:
            outfile.write(f'{lab_alias}:{exp.geo}\t{exp.title}\t{types[exp.exptype]}\t{exp.bs,}\t{",".join(exp.runs)}\tGEO:{exp.geo}\n')
    with open(outf + '_fqs.tsv', 'w') as outfile:
        for exp in gds.experiments:
            if exp.layout == 'single':  # single end reads
                for run in exp.runs:
                    outfile.write(f'{lab_alias}:{run}_fq\t{exp.title}\tfastq\t \t \t \t{str(exp.length)}\t{exp.instr}\tSRA:{run}\n')
            elif exp.layout == 'paired':  # paired end reads
                for run in exp.runs:
                    alias = lab_alias + ':' + run
                    outfile.write(f'{alias}_fq1\t{exp.title}\tfastq\t1\tpaired with\t{alias}_fq2\t{str(exp.length)}\t{exp.instr}\tSRA:{run}\n')
                    outfile.write(f'{alias}_fq2\t{exp.title}\tfastq\t2\tpaired with\t{alias}_fq1\t{str(exp.length)}\t{exp.instr}\tSRA:{run}\n')
    with open(outf + '_bs.tsv', 'w') as outfile:
        for biosample in gds.biosamples:
            outfile.write(f'{lab_alias}:{biosample.acc}\t{biosample.description}\tBioSample:{biosample.acc}\n')


def append_xlsx_rows_unformatted(sheet, content_dict):
    '''
        Appends content as a row to the given xlsx sheet. The sheet must have
        column names in the first row. The content to append must be formatted
        as a dictionary with column:value where column matches a column name
        from the sheet and value is a string. Dictionary keys that do not match
        the sheet columns are ignored. Sheet columns for which no key is present
        in the dictionary will get an empty string. The row is appended at the
        bottom of the sheet.
    '''
    fields = list_column_titles(sheet)
    content_row = ['' for i in range(len(fields))]
    for field in content_dict:
        if field in fields:
            content_row[fields.index(field)] = content_dict[field]
    sheet.append(content_row)
    return sheet


def write_experiments(sheet_name, experiments, alias_prefix, file_dict, inbook, outbook, types=type_dict):
    '''
    Writes relevant Experiment object attributes to an Experiment sheet.
    Possible sheet types: ExperimentSeq, ExperimentHiC, ExperimentRepliseq,
                          ExperimentAtacseq, ExperimentDamid, ExperimentChiapet
    Writes alias, description, biosample, files, dbxrefs, and experiment_type
    fields, as appropriate.
    '''
    sheet = outbook[sheet_name]
    print(f"Writing {sheet_name} sheet...")
    for exp in experiments:
        content_dict = {
            'aliases': alias_prefix + ':' + exp.geo,
            'description': exp.title,
            'dbxrefs': 'GEO:' + exp.geo
        }
        if 'Biosample' in inbook.sheetnames:
            content_dict['*biosample'] = alias_prefix + ':' + exp.bs
        if 'FileFastq' in inbook.sheetnames:
            content_dict['files'] = ','.join(file_dict[exp.geo])
        if exp.link:
            content_dict['dbxrefs'] += ', SRA:' + exp.link
        if exp.exptype in types:
            content_dict['*experiment_type'] = types[exp.exptype]
        sheet = append_xlsx_rows_unformatted(sheet, content_dict)
    return outbook


def experiment_type_compare(sheetname, expt_list, geo, inbook):
    '''
    For a given experiment type, looks for that type in workbook sheets and compares
    to experiment types of GEO record/dataset. If present in both, will write
    experiments to file; if either is missing, will print an warning message.
    '''
    expt_dict = {'Atacseq': 'ATAC-seq', 'Damid': 'DamID', 'Chiapet': 'ChIA-PET',
                 'Seq': 'ChIP-seq, RNA-seq, SPRITE, or TSA-seq'}
    expt_name = sheetname[10:] if sheetname[10:] not in expt_dict.keys() else expt_dict[sheetname[10:]]
    type_name = sheetname[10:] if sheetname != 'ExperimentSeq' else '<experiment_type>'
    if sheetname in inbook.sheetnames and expt_list:
        return True
    elif sheetname in inbook.sheetnames and not expt_list:
        print(f"\nNo {expt_name} experiments parsed from {geo}.")
        print(f"If all samples are known to be {expt_name} experiments,")
        print(f"this script can be rerun using -t {type_name}")
    elif sheetname not in inbook.sheetnames and expt_list:
        print(f"\n{expt_name} experiments found in {geo} but no {sheetname} sheet")
        print("present in workbook.",
              expt_name if sheetname != 'ExperimentSeq' else 'These',
              "experiments will not be written to file.")
    return False


def modify_xlsx(geo, infile, outfile, alias_prefix, experiment_type=None, types=valid_types):
    '''
    Looks up a GEO Series record, parses it along with its associated SRA and
    BioSample records, and writes relevant attributes to the specified file. An
    excel template workbook must be specified, and for each type of metadata
    object, will look for the relevant sheet in the workbook. If sheet is absent
    these won't get written.
    '''
    gds = get_geo_metadata(geo, experiment_type)
    if not gds:
        return
    book = openpyxl.load_workbook(infile)

    # copy book to outbook
    outbook = openpyxl.Workbook()
    outbook.remove(outbook.active)  # removes the empty sheet created by default named Sheet
    for sheet_name in book.sheetnames:
        new_sheet = copy_xlsx_sheet(book[sheet_name], outbook, sheet_name)[0]

    get_organisms = requests.get('https://data.4dnucleome.org/search/?type=Organism&frame=object&format=json')
    organisms = [d['scientific_name'] for d in get_organisms.json()['@graph'] if d.get('scientific_name')]
    bs_to_write = [bs.acc for bs in gds.biosamples if bs.organism in organisms]
    exp_to_write = [exp for exp in gds.experiments if exp.public and exp.bs in bs_to_write]

    exp_sheets = [name for name in book.sheetnames if name.startswith('Experiment')]
    if len(exp_sheets) > 0:
        # looks for each experiment type in parsed data
        # then looks for relevant worksheet in excel template
        # writes experiments to file if both present
        hic_expts = [exp for exp in exp_to_write if exp.exptype.startswith('hic')
                     or exp.exptype.startswith('dnase hic')]
        seq_expts = [exp for exp in exp_to_write if exp.exptype in
                     ['chipseq', 'rnaseq', 'tsaseq'] or 'sprite' in exp.exptype]
        atac_expts = [exp for exp in exp_to_write if exp.exptype == 'atacseq']
        rep_expts = [exp for exp in exp_to_write if exp.exptype == 'repliseq']
        dam_expts = [exp for exp in exp_to_write if exp.exptype.startswith('damid')]
        cap_expts = [exp for exp in exp_to_write if exp.exptype == 'capturec']
        chia_expts = [exp for exp in exp_to_write if exp.exptype in ['chiapet', 'placseq']]
        sheet_types = {'HiC': hic_expts, 'Seq': seq_expts, 'Damid': dam_expts,
                       'Atacseq': atac_expts, 'Repliseq': rep_expts,
                       'CaptureC': cap_expts, 'Chiapet': chia_expts}

        keep = []
        keep_keys = []
        for key in sheet_types.keys():
            if experiment_type_compare('Experiment' + key, sheet_types[key], geo, book):
                keep += sheet_types[key]
                keep_keys.append(key)

    if 'Biosample' in book.sheetnames:
        bs_sheet = outbook['Biosample']
        print("Writing Biosample sheet...")
        for entry in gds.biosamples:
            if entry.acc in bs_to_write and entry.acc in [item.bs for item in keep]:
                # write each Biosample object to file
                content_dict = {
                    'aliases': alias_prefix + ':' + entry.acc,
                    'description': entry.description,
                    'dbxrefs': 'BioSample:' + entry.acc
                }
                if 'BiosampleCellCulture' in book.sheetnames:
                    content_dict['cell_culture_details'] = alias_prefix + ':' + entry.acc + '-cellculture'
                bs_sheet = append_xlsx_rows_unformatted(bs_sheet, content_dict)

    if 'BiosampleCellCulture' in book.sheetnames:
        bcc_sheet = outbook['BiosampleCellCulture']
        print("Writing BiosampleCellCulture sheet...")
        for entry in gds.biosamples:
            if entry.acc in bs_to_write and entry.acc in [item.bs for item in keep]:
                # generate aliases for BiosampleCellCulture sheet
                content_dict = {
                    'aliases': alias_prefix + ':' + entry.acc + '-cellculture'
                }
                bcc_sheet = append_xlsx_rows_unformatted(bcc_sheet, content_dict)

    file_dict = {}
    if 'FileFastq' in book.sheetnames:
        fq_sheet = outbook['FileFastq']
        print("Writing FileFastq sheet...")
        for entry in gds.experiments:
            if entry.bs in bs_to_write and entry in keep:
                file_dict[entry.geo] = []
                for run in entry.runs:
                    # write information about SRA runs to file -
                    # assumes they will be downloaded as fastq files
                    content_dict = {
                        '*file_format': 'fastq',
                        'read_length': entry.length,
                        'instrument': entry.instr,
                        'dbxrefs': 'SRA:' + run
                    }
                    if entry.layout.lower() == 'paired':
                        fq1 = alias_prefix + ':' + run + '_1_fq'
                        fq2 = alias_prefix + ':' + run + '_2_fq'
                        file_dict[entry.geo] += [fq1, fq2]
                        content_dict_fq1 = {k: v for k, v in content_dict.items()}
                        content_dict_fq1['aliases'] = fq1
                        content_dict_fq1['paired_end'] = '1'
                        content_dict_fq1['related_files.relationship_type'] = 'paired with'
                        content_dict_fq1['related_files.file'] = fq2
                        fq_sheet = append_xlsx_rows_unformatted(fq_sheet, content_dict_fq1)
                        content_dict_fq2 = {k: v for k, v in content_dict.items()}
                        content_dict_fq2['aliases'] = fq2
                        content_dict_fq2['paired_end'] = '2'
                        fq_sheet = append_xlsx_rows_unformatted(fq_sheet, content_dict_fq2)

                    elif entry.layout.lower() == 'single':
                        fq_0 = alias_prefix + ':' + run + '_fq'
                        file_dict[entry.geo] += [fq_0]
                        content_dict['aliases'] = fq_0
                        fq_sheet = append_xlsx_rows_unformatted(fq_sheet, content_dict)

                    else:
                        raise ValueError("Invalid value for layout. Layout must be 'single' or 'paired'.")

    if len(exp_sheets) > 0:
        for key in keep_keys:
            outbook = write_experiments('Experiment' + key, sheet_types[key],
                                        alias_prefix, file_dict, book, outbook)

        other_organisms = [exp.geo for exp in gds.experiments if exp.bs not in bs_to_write]
        if other_organisms:
            print('\nThe following accessions were from non-4DN organisms and were not written to file:')
            print('\n'.join(other_organisms))
        other = [exp for exp in gds.experiments if exp.exptype not in types]
        skip = ['bisulfiteseq', 'groseq', 'smartseq', '4cseq']
        skipped = [e for e in other if e.exptype in skip]
        if skipped:
            print('\nThe following accessions had non-4DN experiment types and were not written to file:')
            print('\n'.join([item.geo for item in skipped]))
            other = [e for e in other if e.exptype not in skip]
        if other:
            if len(other) + len(skipped) == len(gds.experiments):
                print(f"\nExperiment types of dataset could not be parsed. {', '.join(exp_sheets)} sheet not written")
            else:
                print("\nThe following accessions had experiment types that could not be parsed:")
                for item in other:
                    print(item.geo)
            print("If these samples are of a single known experiment type,",
                  "this script can be rerun using -t <experiment_type>")

    outbook.save(outfile)
    print(f"\nWrote file to {outfile}.")
    return


def main(types=valid_types, descr=description, epilog=epilog):  # pragma: no cover
    parser = argparse.ArgumentParser(description=descr, epilog=epilog,
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('geo_accession', help="GEO accession", action="store")
    parser.add_argument('-i', '--infile', help="Input xlsx file - blank submit4dn workbook",
                        action="store", required=True)
    parser.add_argument('-o', '--outfile', help="Output xlsx file - default output \
                        filename will be GEO accession with xlsx extension",
                        default='', action="store")
    parser.add_argument('-a', '--alias', help="Alias prefix, default is '4dn-dcic-lab'",
                        action="store", default="4dn-dcic-lab")
    parser.add_argument('-e', '--email', help="Email address to use NCBI Entrez",
                        action="store", default="")
    parser.add_argument('-k', '--apikey', help="API Key for NCBI Entrez",
                        action="store", default="")
    parser.add_argument('-t', '--type', help="Optional: type of experiment in series. \
                        By default experiment type is parsed from SRA records, but \
                        this option is useful when parsing isn't straightforward. \
                        Accepted types: HiC, ChIP-seq, RNA-seq, TSA-seq, ATAC-seq, DamID, Repliseq, \
                        DNase HiC, Capture HiC, PLAC-seq, DNA SPRITE, RNADNA SPRITE. \
                        Note that only one type may be specified, so make sure GEO Series \
                        doesn't include multiple experiment types.",
                        action="store", default=None)
    args = parser.parse_args()
    out_file = args.outfile if args.outfile else args.geo_accession + '.xlsx'
    if args.type and args.type.lower().replace('-', '') not in types:
        print(f"\nError: {args.type} not a recognized type\n")
        parser.print_help()
        sys.exit()
    if not args.email:
        Entrez.email = input('Enter email address to use NCBI Entrez: ')
    else:
        Entrez.email = args.email
    if args.apikey:
        Entrez.api_key = args.apikey
    modify_xlsx(args.geo_accession, args.infile, out_file, args.alias, experiment_type=args.type)


if __name__ == '__main__':  # pragma: no cover
    main()



#!/usr/bin/env python3
''' This can be used to directly patch items using a json formatted payload
    the input file - 1 line per item uuid<tab>json payload
    useful for patching attributions
'''

import sys
import argparse
import json
from datetime import datetime
from dcicutils.ff_utils import patch_metadata
from functions.script_utils import create_ff_arg_parser, authenticate


def get_args():  # pragma: no cover
    parser = argparse.ArgumentParser(
        description='Given a file of uuid<tab>json items (one per line) patch item in db',
        parents=[create_ff_arg_parser()],
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )

    parser.add_argument('infile',
                        help="the datafile containing object data to import")
    args = parser.parse_args()
    return args


def main():  # pragma: no cover
    start = datetime.now()
    print(str(start))
    args = get_args()
    auth = authenticate(key=args.key, keyfile=args.keyfile, env=args.env)

    # assumes a single line corresponds to json for single term
    if not args.dbupdate:
        print("DRY RUN - use --dbupdate to update the database")
    with open(args.infile) as items:
        for i in items:
            [iid, payload] = [t.strip() for t in i.split('\t')]
            payload = json.loads(payload)
            if args.dbupdate:
                e = patch_metadata(payload, iid, auth)
            else:
                print("DRY RUN\n\tPATCH: ", iid, " TO\n", payload)
                e = {'status': 'success'}

            status = e.get('status')
            if status and status == 'success':
                print(status)
            else:
                print('FAILED', e)

    end = datetime.now()
    print("FINISHED - START: ", str(start), "\tEND: ", str(end))


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': '.'}

packages = \
['scripts']

package_data = \
{'': ['*']}

install_requires = \
['Biopython==1.76',
 'GEOparse>=2.0.1,<3.0.0',
 'dcicutils>=7.4.0,<8.0.0',
 'jupyterlab==3.2.8',
 'openpyxl>=3.0.9,<4.0.0']

entry_points = \
{'console_scripts': ['fetch-items = scripts.item_fetcher:main']}

setup_kwargs = {
    'name': 'dcicwrangling',
    'version': '2.1.0.9b7',
    'description': 'Scripts and Jupyter notebooks for 4DN wrangling',
    'long_description': "# DCIC-wrangling\n\nThis is a collection of scripts and Jupyter notebooks that can be helpful when performing many data wrangling tasks.  Most of the tools are specific to 4DN Nucleome wrangling needs, however may be modified to be more generally useful for certain tasks.\n\n## Install\n\nPackaged with `poetry` can be installed using `make`, `poetry` or `pip`.\n\nFrom the dcicwrangling directory - `make build`\n\nIf you already have poetry installed - `poetry install`\n\nOr to pip install from PyPi - `pip install dcicwrangling`\n\nAll dependencies are installed by default - if for some reason you don't want to install `pytest` packages or `invoke` (used to launch notebooks) you can do `poetry install --no-dev` - not recommended.\n\n## Usage\n\n### Jupyter notebooks\nThere are a collection of commonly used jupyter notebooks in the `notebooks/useful_notebooks` directory.  You can start a jupyter notebook server locally using `invoke notebook` from the top level directory.  This should launch the server and open a browser page where the notebooks can be accessed.\n\n**IMPORTANT!** - You should create your own folder in the `notebooks` directory named `Yourname_scripts`.  This folder is where you should create, access and run your notebooks.  If you want to start with one of the notebooks in the useful_notebooks directory please create a copy and move it to your own folder.  This keeps the repository clean and organized.  Please **DO NOT** run notebooks in the useful_notebooks directory and commit the results to the repository.\n\n### Scripts\n\nThe scripts directory contains some useful command line scripts.  They can be run from the top level directory using `python scripts/script_name --options`.  Using `--help` shows available options.  In general, modified versions and bespoke scripts should not be committed to the repository - or alternatively committed to a separate non-master branch.\n\nAs scripts are developed and refined `tool.poetry.scripts` directives can be added to facilitate script usage - see `pyproject.toml` file example.\n",
    'author': '4DN-DCIC Team',
    'author_email': 'support@4dnucleome.org',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/4dn-dcic/dcicwrangling',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.1,<3.9',
}


setup(**setup_kwargs)



import sys
import argparse
from dcicutils.ff_utils import get_authentication_with_server, get_metadata
from functions import script_utils as scu


def get_excluded(exclude_types=None, include_types=None):
    # there are some types that we almost certainly want excluded
    exclude = ['User', 'Lab', 'Award', 'OntologyTerm', 'Ontology', 'Organism', 'Publication']
    if exclude_types is not None:
        exclude.extend(exclude_types)
    if include_types is not None:
        exclude = [ty for ty in exclude if ty not in include_types]
    return list(set(exclude))


def is_released(itemid, auth):
    item = get_metadata(itemid, auth)
    if item.get('status'):
        if item['status'] == 'released':
            return True
    return False


def get_args(args):
    parser = argparse.ArgumentParser(
        description='Add a tag to provided items (and optionally their children)',
        parents=[scu.create_input_arg_parser(), scu.create_ff_arg_parser()],
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument('--types2exclude',
                        nargs='+',
                        help="List of Item Types to Explicitly Exclude getting - \
                        by default 'User', 'Lab', 'Award', 'OntologyTerm', 'Ontology', \
                        and 'Organism' are excluded - add others using this argument")
    parser.add_argument('--types2include',
                        nargs='+',
                        help="List of Item Types (that are usually excluded - see \
                        --types2exclude help) that you want to include")
    parser.add_argument('--no_children',
                        nargs='+',
                        help="List of Item types not to get children of")
    parser.add_argument('--include_released',
                        default=False,
                        action='store_true',
                        help='Normally released items are skipped \
                        - this flag includes them in the final list')
    args = parser.parse_args(args)
    return args


def main():
    args = get_args(sys.argv[1:])
    auth = scu.authenticate(key=args.key, keyfile=args.keyfile, env=args.env)
    itemids = scu.get_item_ids_from_args(args.input, auth, args.search)
    excluded_types = get_excluded(args.types2exclude, args.types2include)
    no_child = ['Publication', 'Lab', 'User', 'Award']  # default no_childs
    if args.no_children:
        no_child.extend(args.no_children)
        no_child = list(set(no_child))

    all_linked_ids = []
    # main loop through the top level item ids
    for itemid in itemids:
        linked = scu.get_linked_items(auth, itemid, {})
        if excluded_types is not None:
            linked = scu.filter_dict_by_value(linked, excluded_types, include=False)
        ll = [(k, linked[k]) for k in sorted(linked, key=linked.get)]
        for i, t in ll:
            suff = ''
            if i == itemid:
                suff = '\tINPUT'
            if is_released(i, auth):
                suff = '\tRELEASED' + suff
                if not args.include_released:
                    print(i, '\t', t, '\tSKIPPING', suff)
                    continue
            if i not in all_linked_ids:
                all_linked_ids.append(i)
            else:
                suff = suff + '\tSEEN'
            print(i, '\t', t, suff)
    for a in all_linked_ids:
        print(a)


if __name__ == '__main__':  # pragma: no cover
    main()



#!/usr/bin/env python3
'''
copy files from raw file bucket to processed file bucket
'''
import sys
import argparse
import boto3
from dcicutils.ff_utils import get_metadata
from functions import script_utils as scu


def get_args(args):
    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter,
        parents=[scu.create_ff_arg_parser(), scu.create_input_arg_parser()],
    )
    args = parser.parse_args()
    return args


def main():  # pragma: no cover
    # initial set up
    args = get_args(sys.argv[1:])
    auth = scu.authenticate(key=args.key, keyfile=args.keyfile, env=args.env)
    print("Working on {}".format(auth.get('server')))
    # bucket addresses
    ff_health = get_metadata('/health', auth)
    source_bucket = ff_health['file_upload_bucket']
    target_bucket = ff_health['processed_file_bucket']
    s3 = boto3.resource('s3')

    # get the uuids for the files
    uids = scu.get_item_ids_from_args(args.input, auth, args.search)
    files2copy = [get_metadata(uid, auth).get('upload_key') for uid in uids]

    for file_key in files2copy:
        copy_source = {'Bucket': source_bucket, 'Key': file_key}
        if args.dbupdate:
            try:
                s3.meta.client.copy(copy_source, target_bucket, file_key)
            except Exception:
                print('Can not find file on source', file_key)
                continue
        else:
            print("DRY RUN!")
        print(file_key + ' cp from ' + source_bucket + ' to ' + target_bucket)
        # print('{} file copied'.format(file_key))


if __name__ == '__main__':
    main()
